package fsl.depict.netcdf;

/**
 * The formatting rule, encapsulating how a particular Netcdf
 * parameter is to be displayed.
 *
 * @author Christopher Golden
 */
public class FormattingRule {


	// Public Static Constants

	/**
	 * North placement.
	 */
	public static final int NORTH = 0;

	/**
	 * Northeast placement.
	 */
	public static final int NORTHEAST = 1;

	/**
	 * East placement.
	 */
	public static final int EAST = 2;

	/**
	 * Southeast placement.
	 */
	public static final int SOUTHEAST = 3;

	/**
	 * South placement.
	 */
	public static final int SOUTH = 4;

	/**
	 * Southwest placement.
	 */
	public static final int SOUTHWEST = 5;

	/**
	 * West placement.
	 */
	public static final int WEST = 6;

	/**
	 * Northwest placement.
	 */
	public static final int NORTHWEST = 7;

	/**
	 * Center placement.
	 */
	public static final int CENTER = 8;


	// Protected Variables

	/**
	 * Identifier.
	 */
	protected String id;

	/**
	 * Placement.
	 */
	protected int placement;


	// Public Constructors

	/**
	 * Construct a standard instance.
	 *
	 * @param id         Identifier of the parameter.
	 * @param placement  Placement around the center
	 *                   point.
	 */
	public FormattingRule(String id, int placement) {
		this.id = id;
		this.placement = placement;
	}


	// Public Methods

	/**
	 * Get the identifier.
	 *
	 * @return Identifier.
	 */
	public String getIdentifier() {
		return id;
	}

	/**
	 * Get the placement around the center point.
	 *
	 * @return Placement around the center point.
	 */
	public int getPlacement() {
		return placement;
	}
}
