#!/bin/csh 
#
# runPUFF.csh
#
# FUNCTION:
#
# This script is called from an FXC Client computer to launch the PUFF generator
# AWIPS program.
#
# WARNING WARNING WARNING:
#
# ALL echo output that IS NOT REDIRECTED is displayed on the FXC CLIENT SYSTEM
# This is an intentional messaging system to FXC CLIENTs
# Currently this only applies to Standard Out streams.
#
#
# ASSUMPTIONS:
#       * Version 2.0 of the puff software is installed
#	* This script is run under the fxa account.
#       * The following ENVIRONMENT VARIABLES are set in the user shell
#		FXC_HOME
#		FXA_HOME
#		LOG_DIR
#		FXA_DATA
#
# INPUTS:
#		USAGE: runPUFF.csh <arg1> <arg2>.....<arg20>
#
# OUTPUT:
#		RUN_PUFF_CSH_LOG_FILE	
#
# SUCCESS:  exit(0) if runPUFF.csh ran successfully
#           exit(1) if not enough arguments provided, or a runPUFF process
#           is already executing.
# 
# NOTE for FXC Usage:  If you are using this script in conjunction with the FXC
#                      PUFF GUI Tool, writing to the log status file
#                      is very important.  When data is written to the log status
#                      file, the PUFF GUI tool is notified that PUFF
#                      model data is available.  Do not write to the log status
#                      file unless model data really is finished processing and
#                      available.
#
# HISTORY:
#
# v1.6 07.29.2005 GWP Added logic to remove ENSEMBLE text first guess files prior
#                     to running again.
# v1.5 11.03.2004 JTF Added logic to remove puff Raw and puff output files prior
#                     to running again.
# v1.4 10.24.2004 JTF BUG and CODING FIXES and Logging all output from this script
#		      Added necessary comments, and assumptions.
# v1.3 10.23.2004 LG  see cvs log notes
# v1.2 10.22.2004 LG  Added logic to check for an already running version
#                     of this script.  Modified the args required for TIME
#                     variable.
# v1.1 10.04.2004 LG  Modified Command arguments.  Added output and input path 
#                     variables. Modified runPuff command line.
#                     Modified binary path and log file path + name.
# v1.0 09.23.2004 JTF Original Version
#
# ************************************************************************

# CONFIGURABLE VARIABLES
set RUN_PUFF_CSH_LOG_FILE = $LOG_DIR/runPUFF_csh.LOG

set PUFF_BINARY_PATH = $FXA_HOME/bin 
set PUFF_GRID_CONFIG = $FXA_HOME/data/PUFFConfig.txt
set SITE = "FSL"
set DTMINS = "5" 

set CURRENT_RUNTIME_DIR = `pwd`

echo " " >> $RUN_PUFF_CSH_LOG_FILE
echo " " >> $RUN_PUFF_CSH_LOG_FILE
echo "----------------------------------------------------------------------------" >> $RUN_PUFF_CSH_LOG_FILE
echo "STARTING THE runPUFF.csh SCRIPT" >> $RUN_PUFF_CSH_LOG_FILE 
echo "STARTING THE runPUFF.csh SCRIPT"
echo `date` >> $RUN_PUFF_CSH_LOG_FILE
echo `date`
echo "----------------------------------------------------------------------------" >> $RUN_PUFF_CSH_LOG_FILE
echo "--------------------------------------------------------"

if ( !(-e $PUFF_BINARY_PATH/puff) ) then
      echo "Error, Can not run puff: No ${PUFF_BINARY_PATH}/puff Binary installed...." >> $RUN_PUFF_CSH_LOG_FILE
      echo "Error, Can not run puff: No ${PUFF_BINARY_PATH}/puff Binary installed...."
      exit(1)
      endif


if ($#argv < 22 ) then
    echo "" >> $RUN_PUFF_CSH_LOG_FILE
    echo "USAGE: \!\!\!\!" >> $RUN_PUFF_CSH_LOG_FILE
    echo "YOU MUST supply the following 23 arguments:" >> $RUN_PUFF_CSH_LOG_FILE
    echo "" >> $RUN_PUFF_CSH_LOG_FILE
    echo "runPUFF.csh <VOLCANO NAME> <ERUPT YEAR> <ERUPT MONTH> <ERUPT DAY> <ERUPT HOUR>" >> $RUN_PUFF_CSH_LOG_FILE
    echo "            <LMS> <SIGMA> <PHIDIST> <SIMHRS> <ERUPTHRS> <SAVEHRS>" >> $RUN_PUFF_CSH_LOG_FILE
    echo "            <NUMPARTS> <PLUME HEIGHT> <PLUME BOTTOM> <PLUME WIDTH> <PLUME THICK>" >> $RUN_PUFF_CSH_LOG_FILE
    echo "            <HORIZ DIFFUSION>  <VERTICAL DIFFUSION> <PLUME SHAPE> <RESTART FILE> <SITE>" >> $RUN_PUFF_CSH_LOG_FILE
    echo "            <DTMINS> <ADDITONAL USERARGS>" >> $RUN_PUFF_CSH_LOG_FILE
    echo "i.e. runPUFF.csh '"spurr"' 2004 10 14 00 -5 1 1 24 24 6 3000 16000 0 0 3 20000 10 linear none none none none" >> $RUN_PUFF_CSH_LOG_FILE
    echo "" >> $RUN_PUFF_CSH_LOG_FILE
    echo "********************************************************************" >> $RUN_PUFF_CSH_LOG_FILE
    echo "" >> $RUN_PUFF_CSH_LOG_FILE
    exit(1)
else

      # Make sure puff knows where the volcano list is.
      setenv PUFF_VOLCANO_LIST $FXC_HOME/fsl/data/config/puffVolcanos.txt

      # Valid Command Line Arguments       #Map command  to puff options in puff manual
      #set VOLCANO = "$argv[1]"             #volcano name
      # Replaces all #^#+# character sequences in the volcano name with a space.
      # This was a workaround for the FXC Script server calling this script
      # with a single command string instead of a cmdArray.
      set VOLCANO = `echo $argv[1] | sed 's/#^#+#/ /g'`
      echo "ARGV[1]: $VOLCANO" >> $RUN_PUFF_CSH_LOG_FILE
      set TIME = "$argv[2] $argv[3] $argv[4] ${argv[5]}:00"       #eruptDate  
      echo "ARGV[2]-ARGV[5]: $TIME" >> $RUN_PUFF_CSH_LOG_FILE
 
      # OTHER PUFF OPTIONS 
      set LMS = "$argv[6]"                 #ashLogMean -- 
      echo "ARGV[6]: $LMS" >> $RUN_PUFF_CSH_LOG_FILE
      set SIGMA = "$argv[7]"               #ashLogSdev -- 
      echo "ARGV[7]: $SIGMA" >> $RUN_PUFF_CSH_LOG_FILE
      set PHIDIST = "$argv[8]"              #phiDist -- not used in version 2.0
      echo "ARGV[8]: $PHIDIST" >> $RUN_PUFF_CSH_LOG_FILE
      set SIMHRS = "$argv[9]"              #runHours
      echo "ARGV[9]: $SIMHRS" >> $RUN_PUFF_CSH_LOG_FILE
      set ERUPTHRS = "$argv[10]"           #eruptHours
      echo "ARGV[10]: $ERUPTHRS" >> $RUN_PUFF_CSH_LOG_FILE
      set SAVEHRS = "$argv[11]"            #saveHours
      echo "ARGV[11]: $SAVEHRS" >> $RUN_PUFF_CSH_LOG_FILE 
      set NUMPARTS = "$argv[12]"           #nAsh
      echo "ARGV[12]: $NUMPARTS" >> $RUN_PUFF_CSH_LOG_FILE
      set PHEIGHT = "$argv[13]"            #plumeMax
      echo "ARGV[13]: $PHEIGHT" >> $RUN_PUFF_CSH_LOG_FILE
      set PBOTTOM = "$argv[14]"            #plumeMin
      echo "ARGV[14]: $PBOTTOM" >> $RUN_PUFF_CSH_LOG_FILE
      set PWIDTH = "$argv[15]"             #plumeHwidth
      echo "ARGV[15]: $PWIDTH" >> $RUN_PUFF_CSH_LOG_FILE
      set PTHICK = "$argv[16]"             #plumeZwidth--only makes sense for non-linear plume shapes.
      echo "ARGV[16]: $PTHICK" >> $RUN_PUFF_CSH_LOG_FILE
      set HDIFF = "$argv[17]"              #diffuseH
      echo "ARGV[17]: $HDIFF" >> $RUN_PUFF_CSH_LOG_FILE
      set VDIFF = "$argv[18]"              #diffuseZ
      echo "ARGV[18]: $VDIFF" >> $RUN_PUFF_CSH_LOG_FILE
      set PSHAPE = "$argv[19]"             #plumeShape
      echo "ARGV[19]: $PSHAPE" >> $RUN_PUFF_CSH_LOG_FILE
      set RESTARTFILE = "$argv[20]"        #restartFile -- option not used
      echo "ARGV[20]: $RESTARTFILE" >> $RUN_PUFF_CSH_LOG_FILE
      set DTMINS = "$argv[21]"             #dtMins
      echo "ARGV[21]: $DTMINS" >> $RUN_PUFF_CSH_LOG_FILE
      set SITE = "$argv[22]"               #site where puff is running.  I.E. AVO, AAWU, etc...
      echo "ARGV[22]: $SITE" >> $RUN_PUFF_CSH_LOG_FILE
      #set USERARGS = "$argv[23]"           #arguments that the user specifies
      # Replaces all #^#+# character sequences in the user args with a space.
      # This was a workaround for the FXC Script server calling this script
      # with a single command string instead of a cmdArray.
      set USERARGS = `echo $argv[23] | sed 's/#^#+#/ /g'`
      echo "ARGV[23]: $USERARGS" >> $RUN_PUFF_CSH_LOG_FILE

      # If SITE is not set, then set SITE to default value
      if ($SITE == "none") then
        set SITE = "FSL"
        echo "SITE set to default $SITE" >>  $RUN_PUFF_CSH_LOG_FILE
      endif

      # If DTMIN is not set, then set DTMIN to default value defined
      # in PUFF 2.0.0 Technical Manual.
      if ($DTMINS == "none") then
        set DTMINS = "5"
        echo "DTMIN set to default $DTMINS" >>  $RUN_PUFF_CSH_LOG_FILE
      endif 

      # If USERARGS is not set, then set to default csh value 
      if ("$USERARGS" == "none") then
        set EXTRA = ""
        echo "NO Additional User args set" >>  $RUN_PUFF_CSH_LOG_FILE
      endif        
       
      # Configure PATH variables
      set PUFF_INPUTDIR_RUN1  = "$FXA_DATA/puff/Raw/$SITE/AK216"
      set PUFF_INPUTDIR_RUN2  = "$FXA_DATA/puff/Raw/$SITE/AVN"
      set PUFF_INPUTDIR_RUN3  = "$FXA_DATA/puff/Raw/$SITE/UKMET"
      set PUFF_OUTPUTDIR_RUN1 = "$FXA_DATA/puff/netCDF/$SITE/AK216/MesoEta"
      set PUFF_OUTPUTDIR_RUN2 = "$FXA_DATA/puff/netCDF/$SITE/LATLON/AVN"
      set PUFF_OUTPUTDIR_RUN3 = "$FXA_DATA/puff/netCDF/$SITE/LATLON/UKMET"
      set PUFF_ENSEMBLE_RUN   = "$FXA_DATA/puff/netCDF/$SITE/ENSEMBLE"
      set PUFF_BAD_DIR = "$FXA_DATA/puff/Bad/$SITE/"

      # Groups Path Information options
      set PATH_INFO_RUN1 = "-path $PUFF_INPUTDIR_RUN1 -opath $PUFF_OUTPUTDIR_RUN1"
      set PATH_INFO_RUN2 = "-path $PUFF_INPUTDIR_RUN2 -opath $PUFF_OUTPUTDIR_RUN2"
      set PATH_INFO_RUN3 = "-path $PUFF_INPUTDIR_RUN3 -opath $PUFF_OUTPUTDIR_RUN3"

      # Groups Particle Size options
      set PARTICLE_SIZE = "-ashLogMean $LMS -ashLogSdev $SIGMA"

      # Groups Simulation options
      set SIM_HOURS = "-eruptHours $ERUPTHRS -runHours $SIMHRS -saveHours $SAVEHRS"

      set PLUME_SHAPE = "-plumeShape $PSHAPE"   

      # Groups Plume Characteristic options according to the plume shape
      #PLUME_THICK only makes sense for non-linear plume shapes. 
      if ($PSHAPE != "linear") then
         set PLUME_CHARS = "-plumeMax $PHEIGHT -plumeHwidth $PWIDTH -plumeZwidth $PTHICK" 
      else
          set PLUME_CHARS = "-plumeMin $PBOTTOM -plumeMax $PHEIGHT -plumeHwidth $PWIDTH"
      endif
      # Groups Diffusion Coefficient options
      set DIFF_COEFF = "-diffuseH $HDIFF -diffuseZ $VDIFF"


        #Old PS hangin around until new is tested.
        #if ( `ps -ewf | grep -v grep | grep CreatePuffNetCDFs` ==  ) then
        #  if ( `ps -ewf | grep -v grep | grep 'puff -volc'` ==  )  then 

      # Remove any bad files that reside in the site's Bad directory before trying to
      # create new Puff-friendly NetCDF files
      echo "" >> $RUN_PUFF_CSH_LOG_FILE
      echo "Removing any existing files in the BAD puff directory ........." >> $RUN_PUFF_CSH_LOG_FILE 
      rm -f $PUFF_BAD_DIR/* >& /dev/null
      
        
        #This command returns 0 if a match was found, 1 if no match was found.
        #Create the NETCDF Input for PUFF if not already running
        set fstatus = `ps -ewf | grep -v grep | grep CreatePuffNetCDFs`
        if ( $status ) then
             unset fstatus
             set fstatus = `ps -ewf | grep -v grep | grep 'puff -volc'`
             if ( $status)  then
                 #About to create raw puff netcdfs.
		 #Deleting all files in Raw directories.
		 #This is NOT required, just keeping it clean.
		 #This statement returns status 0 if files exist in the directory.
                 echo "" >> $RUN_PUFF_CSH_LOG_FILE
                 echo "Removing any existing files in the Raw puff input directories ........." >> $RUN_PUFF_CSH_LOG_FILE 
                 cd $PUFF_INPUTDIR_RUN1
                 set fnames = `find . -name '*' | grep '.*'`
                 if !($status) then
                     rm -f $PUFF_INPUTDIR_RUN1/* >& /dev/null
                     echo "Removing files from $PUFF_INPUTDIR_RUN1 ........." >> $RUN_PUFF_CSH_LOG_FILE
                     echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
                 endif
                 cd $PUFF_INPUTDIR_RUN2
                 set fnames = `find . -name '*' | grep '.*'`
                 if !($status) then
                     rm -f $PUFF_INPUTDIR_RUN2/* >& /dev/null
                     echo "Removing files from $PUFF_INPUTDIR_RUN2 ........." >> $RUN_PUFF_CSH_LOG_FILE
                     echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
                 endif
                 cd $PUFF_INPUTDIR_RUN3
                 set fnames = `find . -name '*' | grep '.*'`
                 if !($status) then
                     rm -f  $PUFF_INPUTDIR_RUN3/* >& /dev/null
                     echo "Removing files from $PUFF_INPUTDIR_RUN3 ........." >> $RUN_PUFF_CSH_LOG_FILE
                     echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
                 endif
                 cd $CURRENT_RUNTIME_DIR >& /dev/null

                 echo "" >> $RUN_PUFF_CSH_LOG_FILE
                 echo "Executing CreatePuffNetCDFs for $SITE......................" >> $RUN_PUFF_CSH_LOG_FILE
                 echo "Executing CreatePuffNetCDFs for $SITE......................"
                 #set CREATEPUFFNETCDF_STATUS = "0001"
                 $PUFF_BINARY_PATH/CreatePuffNetCDFs $TIME $SITE >>& $RUN_PUFF_CSH_LOG_FILE
                 set CREATEPUFFNETCDF_STATUS = $status

                 #Get the Grids 
                 set grids = (`sed '! p' $PUFF_GRID_CONFIG`)
                 
                 #complete success
                 if ($CREATEPUFFNETCDF_STATUS == 0) then
                    foreach grid ($grids)
  			echo "$grid Grid available for $SITE..."
                        echo "$grid Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
		    end
                 #total failure
                 else if ($CREATEPUFFNETCDF_STATUS == 1) then
                    echo "ERROR Status RETURNED From CreatePuffNetCDFs."  >> $RUN_PUFF_CSH_LOG_FILE
                    echo "NO PUFF DATA Generated: $TIME $SITE" >> $RUN_PUFF_CSH_LOG_FILE 

                    echo "ERROR Status RETURNED From CreatePuffNetCDFs."
                    echo "NO PUFF DATA Generated" 
                    echo "CHECK YOUR TIME and VALUES: $TIME $SITE"
                    echo "EXITING runPUFF script... NO PUFF DATA Generated."
                    exit(1)
                 #GRID 1 error
                 else if ($CREATEPUFFNETCDF_STATUS == 2) then
                    echo "$SITE $grids[2] Grid available for $SITE..."
                    echo "$SITE $grids[2] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                    echo "$SITE $grids[3] Grid available for $SITE..."
                    echo "$SITE $grids[3] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #Grid 2 error
                 else if ($CREATEPUFFNETCDF_STATUS == 4) then
                    echo "$SITE $grids[1] Grid available for $SITE..."
                    echo "$SITE $grids[1] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                    echo "$SITE $grids[3] Grid available for $SITE..."
                    echo "$SITE $grids[3] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #Grid 1 and Grid 2 errors
                 else if ($CREATEPUFFNETCDF_STATUS == 6) then
                    echo "$SITE $grids[3] Grid available for $SITE..."
                    echo "$SITE $grids[3] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #Grid 3 error
                 else if ($CREATEPUFFNETCDF_STATUS == 8) then
                    echo "$SITE $grids[1] Grid available for $SITE..."
                    echo "$SITE $grids[1] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                    echo "$SITE $grids[2] Grid available for $SITE..."
                    echo "$SITE $grids[2] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #Grid 1 and Grid 3 errors
                 else if ($CREATEPUFFNETCDF_STATUS == 10) then
                    echo "$SITE $grids[2] Grid available for $SITE..."
                    echo "$SITE $grids[2] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #Grid 2 and Grid 3 errors
                 else if ($CREATEPUFFNETCDF_STATUS == 12) then
                    echo "$SITE $grids[1] Grid available for $SITE..."
                    echo "$SITE $grids[1] Grid available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
                 #All bets are off
                 else 
                    echo "ERROR Status RETURNED From CreatePuffNetCDFs."  >> $RUN_PUFF_CSH_LOG_FILE
                    echo "NO PUFF DATA Generated: $TIME $SITE" >> $RUN_PUFF_CSH_LOG_FILE

                    echo "ERROR Status RETURNED From CreatePuffNetCDFs."
                    echo "NO PUFF DATA Generated" 
                    echo "CHECK YOUR TIME and VALUES: $TIME $SITE"
                    echo "EXITING runPUFF script... NO PUFF DATA Generated."
                    exit(1)
                 endif
                 #if ($status) then
                 #     echo "ERROR Status RETURNED From CreatePuffNetCDFs."  >> $RUN_PUFF_CSH_LOG_FILE
                 #     echo "NO PUFF DATA Generated: $TIME $SITE" >> $RUN_PUFF_CSH_LOG_FILE
  
                 #     echo "ERROR Status RETURNED From CreatePuffNetCDFs."
                 #     echo "CHECK YOUR TIME and VALUES: $TIME $SITE"
                 #     echo "EXITING runPUFF script... NO PUFF DATA Generated."
                 #     exit(1)
                 #endif
             else
                 echo "EXITING: puff is already running" >> $RUN_PUFF_CSH_LOG_FILE
                 echo "EXITING: puff is already running"
                 exit (1) 
             endif
        else
             echo "EXITING: CreatePuffNetCDFs already running" >> $RUN_PUFF_CSH_LOG_FILE
             echo "EXITING: CreatePuffNetCDFs already running"
             exit (1)
        endif

        #Old PS hangin around until new is tested.
        #if ( `ps -ewf | grep -v grep | grep 'puff -volc'` ==   ) then 

        #run puff if it's not already running 
        unset fstatus
        set fstatus = `ps -ewf | grep -v grep | grep 'puff -volc'`
        if ( $status ) then      

          #THIS IS REQUIRED.
          #About to run puff ... so deleting all current puff output files.
          #This is required because if a user generates an earlier model run
          #of puff and a more current run of output files exist than the more
          #current output run is what is displayed in FXC.
          #This statement returns status 0 if files exist in the directory.
          echo "" >> $RUN_PUFF_CSH_LOG_FILE
          echo "Removing any existing files in the puff output directories ........." >> $RUN_PUFF_CSH_LOG_FILE 
          #Remove ENSEMBLE first guess files.
          rm -f $PUFF_ENSEMBLE_RUN/* >& /dev/null
          cd $PUFF_OUTPUTDIR_RUN1
          set fnames = `find . -name '*' | grep '.*'`
          #Only remove files for models that ran CreatePuffNetCDF.
          if (!($status) && !(($CREATEPUFFNETCDF_STATUS == 8) || ($CREATEPUFFNETCDF_STATUS == 10) || ($CREATEPUFFNETCDF_STATUS == 12))) then
              rm -f $PUFF_OUTPUTDIR_RUN1/* >& /dev/null
              echo "Removing files from $PUFF_OUTPUTDIR_RUN1 ........." >> $RUN_PUFF_CSH_LOG_FILE
              echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
          endif
          cd $PUFF_OUTPUTDIR_RUN2
          set fnames = `find . -name '*' | grep '.*'`
          if (!($status) && !(($CREATEPUFFNETCDF_STATUS == 2) || ($CREATEPUFFNETCDF_STATUS == 6) || ($CREATEPUFFNETCDF_STATUS == 10))) then
             rm -f $PUFF_OUTPUTDIR_RUN2/* >& /dev/null
              echo "Removing files from $PUFF_OUTPUTDIR_RUN2 ........." >> $RUN_PUFF_CSH_LOG_FILE
              echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
          endif
          cd $PUFF_OUTPUTDIR_RUN3
          set fnames = `find . -name '*' | grep '.*'`
          if (!($status) && !(($CREATEPUFFNETCDF_STATUS == 4) || ($CREATEPUFFNETCDF_STATUS == 6) || ($CREATEPUFFNETCDF_STATUS == 12))) then
              rm -f  $PUFF_OUTPUTDIR_RUN3/* >& /dev/null
              echo "Removing files from $PUFF_OUTPUTDIR_RUN3 ........." >> $RUN_PUFF_CSH_LOG_FILE
              echo "Files: $fnames" >> $RUN_PUFF_CSH_LOG_FILE
          endif
          cd $CURRENT_RUNTIME_DIR >& /dev/null

          # LIVE runPUFF executables for models
          echo "" >> $RUN_PUFF_CSH_LOG_FILE
          echo "Executing puff quietly ......................." >> $RUN_PUFF_CSH_LOG_FILE
          echo "" >> $RUN_PUFF_CSH_LOG_FILE
          echo "Executing puff quietly ......................."

          #RUN PUFF FOR Grid 3 
          if (($CREATEPUFFNETCDF_STATUS == 0) || ($CREATEPUFFNETCDF_STATUS == 2) || ($CREATEPUFFNETCDF_STATUS == 4) || ($CREATEPUFFNETCDF_STATUS == 6)) then    
            $PUFF_BINARY_PATH/puff -volc "$VOLCANO" -eruptDate "$TIME" -nAsh $NUMPARTS $PATH_INFO_RUN1 $SIM_HOURS $PARTICLE_SIZE $PLUME_CHARS $PLUME_SHAPE $DIFF_COEFF -nmc -volcFile $PUFF_VOLCANO_LIST $USERARGS -quiet >>& $RUN_PUFF_CSH_LOG_FILE
            if (($status)) then
               echo "$grids[3] Grid Puff run not available ......................." >> $RUN_PUFF_CSH_LOG_FILE  
               echo "$grids[3] Grid Puff run not available ......................."
            endif
          endif
          #RUN PUFF FOR Grid 1
          if (($CREATEPUFFNETCDF_STATUS == 0) || ($CREATEPUFFNETCDF_STATUS == 4) || ($CREATEPUFFNETCDF_STATUS == 8) || ($CREATEPUFFNETCDF_STATUS == 12)) then 
            $PUFF_BINARY_PATH/puff -volc "$VOLCANO" -eruptDate "$TIME" -nAsh $NUMPARTS $PATH_INFO_RUN2 $SIM_HOURS $PARTICLE_SIZE $PLUME_CHARS $PLUME_SHAPE $DIFF_COEFF -volcFile $PUFF_VOLCANO_LIST $USERARGS -quiet >>& $RUN_PUFF_CSH_LOG_FILE
            if (($status)) then
               echo "$grids[1] Grid Puff run not available ......................." >> $RUN_PUFF_CSH_LOG_FILE   
               echo "$grids[1] Grid Puff run not available ......................."
            endif
          endif
          #RUN PUFF FOR Grid 2
          if (($CREATEPUFFNETCDF_STATUS == 0) || ($CREATEPUFFNETCDF_STATUS == 2) || ($CREATEPUFFNETCDF_STATUS == 8) || ($CREATEPUFFNETCDF_STATUS == 10)) then 
            $PUFF_BINARY_PATH/puff -volc "$VOLCANO" -eruptDate "$TIME" -nAsh $NUMPARTS $PATH_INFO_RUN3 $SIM_HOURS $PARTICLE_SIZE $PLUME_CHARS $PLUME_SHAPE $DIFF_COEFF -volcFile $PUFF_VOLCANO_LIST $USERARGS -quiet >>& $RUN_PUFF_CSH_LOG_FILE
            if (($status)) then
               echo "$grids[2] Grid Puff run not available ......................." >> $RUN_PUFF_CSH_LOG_FILE
               echo "$grids[2] Grid Puff run not available ......................."
            endif
          endif

          # Create the proper output file format for FXC PUFF viewer
          echo "" >> $RUN_PUFF_CSH_LOG_FILE
          echo "Executing MovePuffAsh for $SITE........................" >> $RUN_PUFF_CSH_LOG_FILE
          echo "" >> $RUN_PUFF_CSH_LOG_FILE
          echo "Executing MovePuffAsh for $SITE........................"
          $PUFF_BINARY_PATH/MovePuffAsh $TIME $SIMHRS $SAVEHRS $DTMINS $SITE >>& $RUN_PUFF_CSH_LOG_FILE
          set MOVEPUFFASH_STATUS = $status
                                                                                                                                                          
          #complete success
          if ($MOVEPUFFASH_STATUS == 0) then
             foreach grid ($grids)
                echo "$grid puff data available for $SITE..."
                echo "$grid puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
             end
          #total failure
          else if ($MOVEPUFFASH_STATUS == 1) then
             echo "ERROR Status RETURNED From MovePuffAsh."  >> $RUN_PUFF_CSH_LOG_FILE
             echo "NO PUFF Data Available: $TIME $SITE" >> $RUN_PUFF_CSH_LOG_FILE
                                                                                                                                                          
             echo "ERROR Status RETURNED From MovePuffAsh."
             echo "NO PUFF DATA Available"
             echo "CHECK YOUR TIME and VALUES: $TIME $SITE"
             echo "EXITING runPUFF script... NO PUFF DATA Available."
             exit(1)
          #GRID 1 error
          else if ($MOVEPUFFASH_STATUS == 2) then
             echo "$SITE $grids[2] puff data available for $SITE..."
             echo "$SITE $grids[2] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
             echo "$SITE $grids[3] puff data available for $SITE..."
             echo "$SITE $grids[3] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE  
          #Grid 2 error
          else if ($MOVEPUFFASH_STATUS == 4) then
             echo "$SITE $grids[1] puff data available for $SITE..."
             echo "$SITE $grids[1] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
             echo "$SITE $grids[3] puff data available for $SITE..."
             echo "$SITE $grids[3] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
          #Grid 1 and Grid 2 errors
          else if ($MOVEPUFFASH_STATUS == 6) then
             echo "$SITE $grids[3] puff data available for $SITE..."
             echo "$SITE $grids[3] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
          #Grid 3 error
          else if ($MOVEPUFFASH_STATUS == 8) then
             echo "$SITE $grids[1] puff data available for $SITE..."
             echo "$SITE $grids[1] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
             echo "$SITE $grids[2] puff data available for $SITE..."
             echo "$SITE $grids[2] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
          #Grid 1 and Grid 3 errors
          else if ($MOVEPUFFASH_STATUS == 10) then
             echo "$SITE $grids[2] puff data available for $SITE..."
             echo "$SITE $grids[2] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
          #Grid 2 and Grid 3 errors
          else if ($MOVEPUFFASH_STATUS == 12) then
             echo "$SITE $grids[1] puff data available for $SITE..."
             echo "$SITE $grids[1] puff data available for $SITE..." >> $RUN_PUFF_CSH_LOG_FILE
          #All bets are off
          else
             echo "ERROR Status RETURNED From MovePuffAsh."  >> $RUN_PUFF_CSH_LOG_FILE
             echo "NO PUFF DATA Generated: $TIME $SITE" >> $RUN_PUFF_CSH_LOG_FILE
             echo "ERROR Status RETURNED From MovePuffAsh."
             echo "NO PUFF DATA Generated"
             echo "CHECK YOUR TIME and VALUES: $TIME $SITE"
             echo "EXITING runPUFF script... NO PUFF DATA Generated."
             exit(1)
          endif
        else
          echo "EXITING: puff is already running" >> $RUN_PUFF_CSH_LOG_FILE
          echo "EXITING: puff is already running" 
          exit(1)
        endif
     
endif

echo "NORMAL EXIT OF runPUFF.csh SCRIPT" >> $RUN_PUFF_CSH_LOG_FILE 
echo "NORMAL EXIT of runPUFF script."
echo `date` >> $RUN_PUFF_CSH_LOG_FILE
echo `date`

echo "YOUR REQUESTED PUFF DATA IS Ready for Viewing: $TIME"

exit(0)
