#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# ---------------------------------------------------------------------------
# clientServerSynch
#
#     This script allows an FXC client and serverManager to be started up
#     simultaneously on the local machine, and ensures that when the client
#     process exits, the serverManager will as well.
#
# Author: Chris Golden
# ---------------------------------------------------------------------------

# -- proc -------------------------------------------------------------------
# 
# processOutput
#
# Process output from the specified channel.
#
# Parameters:
#     channelID --  Identifier of the channel that has output ready.
#
# Return value:
#     none.
# 
# ---------------------------------------------------------------------------
proc processOutput { channelID } {
    global client server doneFlag
    
    puts "In processOutput..."

    # If the channel is closing, then both processes should be
    # told to exit.  Otherwise, any output should be echoed to this
    # script's standard output.
    if { [eof $channelID] } {
        catch { close $server }
        catch { close $client }
        set doneFlag 1
    } else {
        while { 1 } {
            set input [gets $channelID]
            if { $input == "" } {
                return
            } elseif { $input == "shutdown" } {
                catch { close $server }
                catch { close $client }
                set doneFlag 1
            }
            puts $input
        }
    }
}

# -- proc -------------------------------------------------------------------
# 
# main
#
# Entry point for the script.
#
# Parameters:
#     none.
#
# Return value:
#     none.
# 
# ---------------------------------------------------------------------------
proc main {} {
    global env client server doneFlag errorCode errorInfo

    # See if another server is already running, and if
    # so, skip starting the server.
    catch {exec ps -ef --cols 400 | grep serverManager | grep ldadWish | grep -v grep | wc -w} serverProcessWords
    if { [string index $serverProcessWords 0] == 0 } {
        set serverProcessWords 0
    }
    #puts "Server: $serverProcessWords"
    #set serverProcessWords [read $psOutput]
    #close $psOutput

    # Set the "script done" flag to false.
    set doneFlag 0
    
    # Start up the serverManager and configure its standard output
    # to be processed by this script.
    if { $serverProcessWords == 0 } {
        cd "$env(FXC_HOME)/fsl/linux"
        #set fullCommand "|serverManager -monitorstdin -hide -private 4 "
        set fullCommand "|serverManager -portrange 1100 1106 -private 5"
        #append fullCommand "-synchfile $env(FXC_HOME)/fsl/data/state/synch.state"
        puts "Server cmd: $fullCommand"
        set server [open $fullCommand r+]
        fconfigure $server -blocking 0 -buffering line
        fileevent $server readable "processOutput $server"
        after 7000
    }
    
    # Find an available port for FXC
    foreach clientPort {1123 1124 1125 1126 1127} {

	set cmd "ps -ef --cols 400 | grep java | grep fsl.main.FXC | grep receiverport=${clientPort} | grep -v grep | wc -l"
        #set cmd "ps -ef --cols 400 | grep IGC_Process | grep -v grep | wc -l"
        #puts "CMD: $cmd"
        catch {eval "exec $cmd"} numClients
        if { [string index $numClients 0] == 0 } {
            set numClients 0
        }
        #set errc $errorCode;set erri $errorInfo
        #puts "Code: $errc"
        #puts "Info: $erri"
        #puts "Num clients = ${numClients} x"
        set numLength [string length $numClients]
        #puts "Length: $numLength"
        if { $numClients > 0 } {
            if { $clientPort == 1127 } {
                puts "All ports in use."
                exit
            } else {
                continue
            }
        } else {
            break
        }
    }
        

    # Start up the FXC client and configure its standard output to
    # be processed by this script.
    cd "$env(FXC_HOME)/fsl/main"
    set fullCommand "|java -Xms512m -Xmx512m -Dfxc.home=$env(FXC_HOME) "
    append fullCommand "-Dfxc.localization=$env(FXA_LOCAL_SITE) "
    # append fullCommand "-Dfxc.synch.files=$env(FXC_HOME)/fsl/data/state/synch.state "
    append fullCommand "-Dfxc.transceiver.receiverport=${clientPort} "
    append fullCommand "-Dfxc.lookandfeel=Metal "

    append fullCommand "-Dfxc.statefile=$env(HOST)${clientPort} "

    append fullCommand "fsl.main.FXC"
    puts "Client cmd: $fullCommand"
    set client [open $fullCommand r+]
    fconfigure $client -blocking 0 -buffering line
    fileevent $client readable "processOutput $client"

    # Wait for the done flag to be set before exiting.
    vwait doneFlag
}

# Start the script.
#set env(FXC_HOME) "/data/local/FXC"
main

### Local Variables: ***
### mode:tcl ***
### End: ***
